#! /usr/bin/env python 
# -*- coding: utf-8 -*-
# by Hans Schou <chlor@schou.dk> 2014-01-15
#
# Syntax example:
#   ./kicad_pcb.py > kicad_pcb.svg

import math, sys

# In the code the term 'fraction' is equal
# to a fraction of a cicle. Fraction = 1 = 360° = 2*pi

OFFSET = {'x':250,'y':170}
toothCount = 8
roundValues = True
RADIUS = 3.0
radius = RADIUS * 3/4
innerRad = RADIUS * 9/32 # aprx ~ 1/4
cornerRadius = 4/RADIUS
angelStart = 1.0/4 + 1.0/(toothCount*2)
angelToothTop = 1.0/(toothCount*3)
angelToothBottom = 1.0/(toothCount*3)
angelToothEdge = (1.0/toothCount - angelToothTop - angelToothBottom) / 2

vectorsTooth = [ {"rad":RADIUS, "ang":angelToothEdge}, {"rad":RADIUS, "ang":angelToothTop}, {"rad":radius, "ang":angelToothEdge} ];

def polarGearLogo():
	angel = angelStart
	p = []
	p.append( dictRadFraction(innerRad, angel) )
	p.append( dictRadFraction(radius, angel) )
	# Draw half tooth space
	angel += angelToothBottom / 2
	p.append( dictRadFraction(radius, angel) )
	n = toothCount - 1 # one tooth is missing on the gear
	while (n > 0):
		# Draw one tooth on the gear
		for pnt in vectorsTooth:
			angel += pnt['ang']
			p.append( dictRadFraction(pnt['rad'], angel) )
		# Draw one space between 2 tooths, or if last point draw a half space
		angel += angelToothBottom / (1 if n > 1 else 2)
		p.append( dictRadFraction(radius, angel) )
		n -= 1
	p.append( dictRadFraction(radius, angel) )
	# Draw inner circle counter clock wise
	p.append( dictRadFraction(innerRad, angel) )
	toothDiv = 3
	n = (toothCount-1)*toothDiv
	while (n > 0):
		angel -= 1.0/(toothCount*toothDiv)
		p.append( dictRadFraction(innerRad, angel) )
		n -= 1
	return p

# Convert radius and fraction to dictionary
# Return dictionary of 'r' and 'f'
def dictRadFraction(rad, fract):
	return { "r": rad, "f": fract }

# Vector in polar coordinate to cartesian coordinate
# Note: The angel is a fraction of a cicle not radians
# Return: Cartesian coordinates dictionary
def polarToCartesian(polar, offset):
	c = []
	for pol in polar:
		c.append( {"x": OFFSET['x'] + offset['x'] + pol['r'] * math.cos(pol['f'] * math.pi*2), "y": OFFSET['y'] + offset['y'] + pol['r'] * math.sin(pol['f'] * math.pi*2)} );
	return c

polar = polarGearLogo()
cartesian = polarToCartesian(polar, {'x':RADIUS,'y':RADIUS})

s = "(kicad_pcb (version 3) (host pcbnew \"(2013-05-10 BZR 4148)-testing\")\n"
s += "\n"
s += "  (general\n"
s += "    (links 0)\n"
s += "    (no_connects 0)\n"
s += "    (area 314.521912 237.051912 327.082088 249.612088)\n"
s += "    (thickness 1.6)\n"
s += "    (drawings 11)\n"
s += "    (tracks 0)\n"
s += "    (zones 0)\n"
s += "    (modules 1)\n"
s += "    (nets 1)\n"
s += "  )\n"
s += "\n"
s += "  (page A4)\n"
s += "  (layers\n"
s += "    (15 F.Cu signal)\n"
s += "    (0 B.Cu signal)\n"
s += "    (16 B.Adhes user)\n"
s += "    (17 F.Adhes user)\n"
s += "    (18 B.Paste user)\n"
s += "    (19 F.Paste user)\n"
s += "    (20 B.SilkS user)\n"
s += "    (21 F.SilkS user)\n"
s += "    (22 B.Mask user)\n"
s += "    (23 F.Mask user)\n"
s += "    (24 Dwgs.User user)\n"
s += "    (25 Cmts.User user)\n"
s += "    (26 Eco1.User user)\n"
s += "    (27 Eco2.User user)\n"
s += "    (28 Edge.Cuts user)\n"
s += "  )\n"
s += "\n"
s += "  (setup\n"
s += "    (last_trace_width 0.254)\n"
s += "    (trace_clearance 0.254)\n"
s += "    (zone_clearance 0.508)\n"
s += "    (zone_45_only no)\n"
s += "    (trace_min 0.254)\n"
s += "    (segment_width 0.2)\n"
s += "    (edge_width 0.1)\n"
s += "    (via_size 0.889)\n"
s += "    (via_drill 0.635)\n"
s += "    (via_min_size 0.889)\n"
s += "    (via_min_drill 0.508)\n"
s += "    (uvia_size 0.508)\n"
s += "    (uvia_drill 0.127)\n"
s += "    (uvias_allowed no)\n"
s += "    (uvia_min_size 0.508)\n"
s += "    (uvia_min_drill 0.127)\n"
s += "    (pcb_text_width 0.3)\n"
s += "    (pcb_text_size 1.5 1.5)\n"
s += "    (mod_edge_width 0.15)\n"
s += "    (mod_text_size 1 1)\n"
s += "    (mod_text_width 0.15)\n"
s += "    (pad_size 1.5 1.5)\n"
s += "    (pad_drill 0.6)\n"
s += "    (pad_to_mask_clearance 0)\n"
s += "    (aux_axis_origin 0 0)\n"
s += "    (visible_elements FFFFFF7F)\n"
s += "    (pcbplotparams\n"
s += "      (layerselection 3178497)\n"
s += "      (usegerberextensions true)\n"
s += "      (excludeedgelayer true)\n"
s += "      (linewidth 0.150000)\n"
s += "      (plotframeref false)\n"
s += "      (viasonmask false)\n"
s += "      (mode 1)\n"
s += "      (useauxorigin false)\n"
s += "      (hpglpennumber 1)\n"
s += "      (hpglpenspeed 20)\n"
s += "      (hpglpendiameter 15)\n"
s += "      (hpglpenoverlay 2)\n"
s += "      (psnegative false)\n"
s += "      (psa4output false)\n"
s += "      (plotreference true)\n"
s += "      (plotvalue true)\n"
s += "      (plotothertext true)\n"
s += "      (plotinvisibletext false)\n"
s += "      (padsonsilk false)\n"
s += "      (subtractmaskfromsilk false)\n"
s += "      (outputformat 1)\n"
s += "      (mirror false)\n"
s += "      (drillshape 1)\n"
s += "      (scaleselection 1)\n"
s += "      (outputdirectory \"\"))\n"
s += "  )\n"
s += "\n"
s += "  (net 0 \"\")\n"
s += "\n"
s += "  (net_class Default \"This is the default net class.\"\n"
s += "    (clearance 0.254)\n"
s += "    (trace_width 0.254)\n"
s += "    (via_dia 0.889)\n"
s += "    (via_drill 0.635)\n"
s += "    (uvia_dia 0.508)\n"
s += "    (uvia_drill 0.127)\n"
s += "    (add_net \"\")\n"
s += "  )\n"
s += "\n"
s += "  (module TO92 (layer F.Cu) (tedit 443CFFD1) (tstamp 52D75002)\n"
s += "    (at 247 173)\n"
s += "    (descr \"Transistor TO92 brochage type BC237\")\n"
s += "    (tags \"TR TO92\")\n"
s += "    (fp_text reference TO92 (at -1.27 3.81) (layer F.SilkS)\n"
s += "      (effects (font (size 1.016 1.016) (thickness 0.2032)))\n"
s += "    )\n"
s += "    (fp_text value BC237 (at -1.27 -5.08) (layer F.SilkS)\n"
s += "      (effects (font (size 1.016 1.016) (thickness 0.2032)))\n"
s += "    )\n"
s += "    (fp_line (start -1.27 2.54) (end 2.54 -1.27) (layer F.SilkS) (width 0.3048))\n"
s += "    (fp_line (start 2.54 -1.27) (end 2.54 -2.54) (layer F.SilkS) (width 0.3048))\n"
s += "    (fp_line (start 2.54 -2.54) (end 1.27 -3.81) (layer F.SilkS) (width 0.3048))\n"
s += "    (fp_line (start 1.27 -3.81) (end -1.27 -3.81) (layer F.SilkS) (width 0.3048))\n"
s += "    (fp_line (start -1.27 -3.81) (end -3.81 -1.27) (layer F.SilkS) (width 0.3048))\n"
s += "    (fp_line (start -3.81 -1.27) (end -3.81 1.27) (layer F.SilkS) (width 0.3048))\n"
s += "    (fp_line (start -3.81 1.27) (end -2.54 2.54) (layer F.SilkS) (width 0.3048))\n"
s += "    (fp_line (start -2.54 2.54) (end -1.27 2.54) (layer F.SilkS) (width 0.3048))\n"
s += "    (pad 1 thru_hole rect (at 1.27 -1.27) (size 1.397 1.397) (drill 0.8128)\n"
s += "      (layers *.Cu *.Mask F.SilkS)\n"
s += "    )\n"
s += "    (pad 2 thru_hole circle (at -1.27 -1.27) (size 1.397 1.397) (drill 0.8128)\n"
s += "      (layers *.Cu *.Mask F.SilkS)\n"
s += "    )\n"
s += "    (pad 3 thru_hole circle (at -1.27 1.27) (size 1.397 1.397) (drill 0.8128)\n"
s += "      (layers *.Cu *.Mask F.SilkS)\n"
s += "    )\n"
s += "    (model discret/to98.wrl\n"
s += "      (at (xyz 0 0 0))\n"
s += "      (scale (xyz 1 1 1))\n"
s += "      (rotate (xyz 0 0 0))\n"
s += "    )\n"
s += "  )\n"
s += "\n"

pntA = cartesian.pop(0)
for pntB in cartesian:
	s += " (segment (start %f %f) (end %f %f) (width 0.254) (layer F.Cu) (net 0))\n" % (pntA['x']+7, pntA['y'], pntB['x']+7, pntB['y'])
	s += " (segment (start %f %f) (end %f %f) (width 0.127) (layer F.Cu) (net 0))\n" % (pntA['x']+7, pntA['y']+7, pntB['x']+7, pntB['y']+7)
	s += " (gr_line (start %f %f) (end %f %f) (angle 90) (layer F.SilkS) (width 0.2))\n" % (pntA['x'], pntA['y']+7, pntB['x'], pntB['y']+7)
	s += " (gr_line (start %f %f) (end %f %f) (angle 90) (layer Cmts.User) (width 0.2))\n" % (pntA['x'], pntA['y'], pntB['x'], pntB['y'])
	pntA = pntB

s += "\n"
s += ")\n"
print s
